<?php
App::uses('EvBasketAppModel', 'EvBasket.Model');
App::uses('CakeSession', 'Model/Datasource');
/**
 * Basket Model
 *
 * @property BasketItem $BasketItem
 * @property BasketTotal $BasketTotal
 */
class Basket extends EvBasketAppModel {

/**
 * The Associations below have been created with all possible keys, those that are not needed can be removed
 */
/**
 * hasMany associations
 *
 * @var array
 */
	public $hasMany = array(
		'BasketItem' => array(
			'className' => 'EvBasket.BasketItem',
			'dependent' => true,
			'cascade' => true
		),
		'BasketTotal' => array(
			'className' => 'EvBasket.BasketTotal',
			'dependent' => true,
			'cascade' => true
		),
		'BasketData' => array(
			'className' => 'EvBasket.BasketData',
			'dependant' => true,
			'cascade' => true
		)
	);

	public $validate = array(
		'hash' => array(
			'unique' => array(
				'rule' => 'isUnique'
			)
		)
	);

/**
 * convience method to get the full basket
 *
 * @param 	string 	$hash 			The hash identifier
 * @param 	array 	$itemContain
 * @return  array|null
 */
	public function getFullBasket($hash, $itemContain, $currencyId = null) {
		if (!$currencyId) {
			$currencyId = CakeSession::read('EvCurrency.currencyId');
		}

		$basket = $this->find(
			'first',
			array(
				'conditions' => array(
					'Basket.hash' => $hash,
					'Basket.currency_id' => $currencyId
				),
				'contain' => array(
					'BasketItem' => $itemContain,
					'BasketTotal' => array(
						'order' => 'BasketTotal.is_grand_total ASC, BasketTotal.sequence ASC'
					),
					'BasketData'
				)
			)
		);

		$this->updateLastActivity($basket);

		if (! empty($basket)) {

			if ((bool)strpos(json_encode($itemContain), 'CustomFields') === true && ! empty($basket['BasketItem'])) {
				$CustomField = EvClassRegistry::init('EvCustomFields.CustomFieldData');

				foreach ($basket['BasketItem'] as $key => $item) {
					$basket['BasketItem'][$key] = $CustomField->formatCfArray($item);
				}
			}

			$basket = $this->formatItemsAndTotals($basket);
		}

		return $basket;
	}

/**
 * Update a baskets last_activity field to the current datetime.
 * Called on every request to get a full basket so we can track activity
 * @param  $basket
 * @return bool|array See Model::save() False on failure or an array of model data on success.
 */
	public function updateLastActivity($basket) {
		$now = date("Y-m-d H:i:s");

		// Only update last_activity if it's been more than a minute since it was last updated.
		// Prevents multiple queries per request etc.
		if (strtotime($basket['Basket']['last_activity']) < strtotime($now) - MINUTE) {
			$this->id = $basket['Basket']['id'];

			return $this->saveField('last_activity', $now);
		}

		return false;
	}

/**
 * format the items and totlas keys to be more easily accessbile
 *
 * @param 	array 	$basket 	The basket array
 * @return 	array 	$basket 	The formated basket array
 */
	public function formatItemsAndTotals($basket) {
		$basket['BasketItem'] = Hash::combine(
			$basket['BasketItem'],
			array(
				'%s.%s',
				'{n}.model',
				'{n}.model_id'
			),
			'{n}'
		);

		// calculate tax for each row item
		$basket['BasketItem'] = $this->BasketItem->calculateTax($basket['BasketItem']);

		$basket['BasketTotal'] = Hash::combine(
			$basket['BasketTotal'],
			'{n}.id',
			'{n}'
		);

		return $basket;
	}

/**
 * check to see if a basket exists at the given hash
 *
 * @param 	string 	$hash 	hash identifier to look for
 * @return 	bool
 */
	public function checkExists($hash, $currencyId = null) {
		if (!$currencyId) {
			$currencyId = CakeSession::read('EvCurrency.currencyId');
		}

		return (bool)$this->find(
			'count',
			array(
				'conditions' => array(
					'Basket.hash' => $hash,
					'Basket.currency_id' => $currencyId
				),
			'	callbacks' => false
			)
		);
	}

/**
 * create a basket row with the given hash
 *
 * @param 	string 	$hash 	hash identifier to save basket ot
 * @return 	bool
 */
	public function createBasket($hash, $currencyId = null, $userId = null) {
		if (empty($currencyId)) {

			$currencyId = CakeSession::read('EvCurrency.currencyId');
			if (empty($currencyId)) {
				$currencyId = CakeSession::read('EvCurrency.id');
			}
		}

		$data = array(
			'Basket' => array(
				'hash' => $hash,
				'currency_id' => $currencyId
			)
		);

		// If set link the user to the basket so the basket can be used across logins
		if ($userId != null) {
			$data['Basket']['user_id'] = $userId;
		}

		$this->clear();

		$result = $this->save($data);

		if ($result !== false) {
			return true;
		}

		return $result;
	}

	/**
	 * Takes two basket hash's, gets the items from the second basket and moves them to the first
	 * @param  string $masterBasketHash    The primary basket that the other basket will be merged into
	 * @param  string $secondaryBasketHash The basket who's items will be merged into the primary basket
	 * @param  bool $removeSecondaryBasket Defaults to false, if set to try it will delete the basket
	 * @return string                      Hash of the new basket
	 */
	public function mergeBaskets($masterBasketHash, $secondaryBasketHash, $removeSecondaryBasket = false) {
		// Get the items in the secordary basket
		$masterBasket = $this->getFullBasket($masterBasketHash, array());
		$secondaryBasket = $this->getFullBasket($secondaryBasketHash, array());

		// If either basket is not returned error
		if (!$masterBasket || !$secondaryBasket) {
			return false;
		}

		// If the baskets don't match currency return false
		if ($masterBasket['Basket']['currency_id'] != $secondaryBasket['Basket']['currency_id']) {
			return false;
		}

		$this->BasketItem->updateAll(
			array(
				'BasketItem.basket_id' => $masterBasket['Basket']['id']
			),
			array(
				'BasketItem.basket_id' => $secondaryBasket['Basket']['id']
			)
		);

		if ($removeSecondaryBasket) {
			$this->delete($secondaryBasket['Basket']['id']);
		}

		return $masterBasketHash;
	}

/**
 * Updates the supplied basket to associate it with the requested user
 *
 * @param string $basketHash The hash of the basket row to update
 * @param string $userId The unique id of the user
 * @return bool
 */
	public function assignUserToBasket($basketHash = '', $userId = '') {
		return $this->updateAll(
			['user_id' => $userId],
			['hash' => $basketHash, 'user_id' => '']
		);
	}

}
